/***********************************************************************/
/***********************************************************************/
/*  $Header$                                                           */
/***********************************************************************/
/*  Give class specific information on entitys */
/***********************************************************************/

#include "db/Entity.h"
#include "db/Info.h"
#include "general/Gather.h"

/***************************************************************/
/*  Reports some info on interface classes  */
/***************************************************************/

static void Entity_report_interfaceClass (o)
	Entity o;
{
	Entity fromclass = get_fromClass (o);
	Entity toclass   = get_toClass (o);

	if (fromclass == NULL && toclass == NULL) {
		writeln_report	("  Can connect to entitys of any class");
	} else {
		if (fromclass == NULL)
			writeln_report	("  One connection to entitys of any class");
		else
			writeln_report	("  One connection to entitys of class [%s]", Entity_title (fromclass));
		if (toclass == NULL)
			writeln_report	("  Other connection to entitys of any class");
		else
			writeln_report	("  Other connection to entitys of class [%s]", Entity_title (toclass));
	}
 	if (isBendingInterface (o))
		writeln_report ("  Connections can be bend");
	else
		writeln_report ("  Connections cannot be be bent");
}

/***************************************************************/
/*  Reports some info on interfaces  */
/***************************************************************/

static void report_interface (o)
	Entity o;
{
	Entity fromclass = get_fromClass (o);
	Entity toclass   = get_toClass (o);
	Entity from      = from_interface (o);
	Entity to        = to_interface (o);

	if (fromclass == NULL && toclass == NULL) {
		writeln_report	("  Can connect to entitys of any class");
	} else {
		if (fromclass == NULL)
			writeln_report	("  One connection to entitys of any class");
		else
			writeln_report	("  One connection to entitys of class [%s]", Entity_title (fromclass));
		if (toclass == NULL)
			writeln_report	("  Other connection to entitys of any class");
		else
			writeln_report	("  Other connection to entitys of class [%s]", Entity_title (toclass));
	}
	if (to != NULL)
		writeln_report	("  Connected to [%s]", Entity_title(to));
	if (from != NULL)
		writeln_report	("  Connected to [%s]", Entity_title(from));
 	if (isBendingInterface (o))
		writeln_report ("  Connections can bend");
	else
		writeln_report ("  Connections cannot be bent");
}

/***********************************************************************/
/* Report underlying system types */
/***********************************************************************/

static Entity underlyingClass (o)
	Entity o;
{
	Entity c;
	FORALL (c, o->class, class)
		if (isUnderlying (c))
			return c;
	return NULL;
}

static char * report_underlyingStuff (entity)
	Entity entity;
{
#define MAXUNDER 128
	Entity class[MAXUNDER];
	Entity o, c;
	Cardinal found = 0, i;

	write_report ("  Underlying protocols/systems: [");
	FOR_ENTITYLIST (o, Entity_subcomponents (entity)) {
		c = underlyingClass (o);
		if (c == NULL)
			continue;
		for (i = 0; i<found; i++)
			if (c == class[i])
				goto out;
		class[found] = c;
		write_report (" %s", TITLE(c));
		found ++;
		if (found == MAXUNDER) {
			sys_error ("Max underlying [%d] exceeded", MAXUNDER);
			return NULL;
		}
		out:;
	}
	if (found == 0)
		writeln_report ("None found]");
	else
		writeln_report ("]");
	return NULL;
}

/***********************************************************************/
/*  Give a status listing on a class  */
/***********************************************************************/

static void report_class (o)
	Entity o;
{
	char * ping;

	PROC ("report_class");

	if (!Entity_isClass (o)) {
		sys_error ("[%s] is not a class", Entity_title (o));
	}

	start_report ("Reporting on [%s] ...", Entity_title(o));
	writeln_report ("Reporting on [%s]", Entity_title(o));
	writeln_report ("  %d subclasses", sizeOf_entityList (Entity_subclasses(o)));
	if (isUnderlying (o))
		writeln_report ("  Instances can will be reported as underlying !!!");
	else
		writeln_report ("  Instances will not be reported as underlying !!!");

	writeln_report ("Pattern for titles is [%s]", get_titleFormat (o));

	if (no_instances_allowed (o))
		writeln_report ("  No instances allowed");
	else {
		writeln_report ("  %d instances", sizeOf_entityList (Entity_get_instances(o)));
		if (has_anonymous_name (o))
			writeln_report ("  Instances are created anonymously");
		else
			writeln_report ("  Instances require a supplied name");
		switch (Entity_typeOf(o)) {
		case o_interface:
			writeln_report ("  Instances will be interfaces");
			break;
		case o_map:
			writeln_report ("  Instances will be maps");
			break;
		case o_system:
			writeln_report ("  Instances will be systems");
			break;
		case o_none:
			/* No instances */
			break;
		default:
			sys_error ("Unknown type %s, [%s]", Entity_typeOf(o));
			break;
		}
	}
	ping = get_ping_instruction (o);
	if (ping == NULL)
		writeln_report ("  Instances cannot be pinged");
	else
		writeln_report ("  Ping description [%s]", ping);

	if (isInterfaceClass (o)) {
		writeln_report ("  Is interface class");
		Entity_report_interfaceClass (o);
	}
}

/***************************************************************/
/*  Status Report */
/***************************************************************/

extern void Entity_status_report (o)
	Entity o;
{
	String ping;
	Cardinal n;

	if (Entity_isClass (o)) {
		report_class (o);
		return;
	}
	start_report ("Reporting on [%s] ...", Entity_title(o));
	writeln_report ("Reporting on [%s]", Entity_title(o));
	writeln_report ("  Reachability [%s]", reachstatus2string (STATUS(o)));
	writeln_report ("  Future [%s]", future2string (FUTURE(o)));
	writeln_report ("  Behaviour [%s]", behaviour2string (BEHAVIOUR(o)));
	writeln_report ("  Capacity %s", capacity2mega_string(CAPACITY(o)));
	writeln_report ("  LoadIn %s",  load2string(LOADIN(o)));
	writeln_report ("  LoadOut %s", load2string(LOADOUT(o)));

	n = sizeOf_entityList (Entity_subcomponents(o));
	if (n == 0)
		writeln_report ("  No subcomponents");
	else
		writeln_report ("  %d subcomponents", n);

	n = sizeOf_entityList (Entity_supercomponents(o));
	if (n == 0)
		writeln_report ("  Not the subcomponent of any other entity");
	else if (n == 1)
		writeln_report ("  Subcomponent of only one other entity");
	else
		writeln_report ("  Subcomponent of %d entitys", n);

	n = sizeOf_entityList (Entity_interfaces(o));
	if (n == 0)
		writeln_report ("  No interfaces");
	else if (n == 1)
		writeln_report ("  One interface", n);
	else
		writeln_report ("  %d interfaces", n);

	if (PINGID(o) == 0)
		writeln_report ("  Not being pinged");
	else
		writeln_report ("  Ping request [%d]", PINGID(o));

	ping = get_ping_instruction (o);
	if (ping == NULL)
		writeln_report ("  Cannot be pinged");
	else
		/* Should make a call to get this value !!! */
		writeln_report ("  Ping instruction [%s]", ping);
	if (CONTACT(o) == NULL)
		writeln_report ("  Contact Unknown");
	else
		writeln_report ("  Contact [%s]", CONTACT(o));

	if (isInterface (o)) {
		writeln_report	("  Is an interface");
		report_interface (o);
	}
	(void) report_underlyingStuff (o);
}

/***********************************************************************/
/*  Give a simple one line status listing on a class  */
/***********************************************************************/

extern char * Entity_simple_class_report (o)
	Entity o;
{
	char report [1024], * type; /* !!! */

	switch (Entity_typeOf(o)) {
	case o_interface:
		type = "[Interfaces]";
		break;
	case o_map:
		type = "[Maps]      ";
		break;
	case o_system:
		type = "[Systems]   ";
		break;
	case o_none:
		type = "[None]      ";
		break;
	default:
		type = "[sys error] ";
		break;
	}
	sprintf (report, "%s %s", type, Entity_title (o));
	return report;
}


/*  Should have an attribute 'contact' in the configuration files */
extern void show_contact (sys)
	Entity sys;
{
	String title, contact;

	if (sys == NULL) {
		sys_error ("NULL entity");
		return;
	}
	title = Entity_title (sys);
	contact = V_STATUS(sys)->contact;
	if (contact) {
		user_inform ("For [%s] contact: %s", title, contact);
	} else {
		user_inform ("Sorry, have no contact information for [%s]", title);
	}
}

/***********************************************************************/
/***********************************************************************/
/*  $Header$                                                           */
/***********************************************************************/
/***********************************************************************/
