/*
 * Copyright (c) 1986 Regents of the University of California
 * All Rights Reserved
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of California at Berkeley. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */

/*
 * Actually, this program is from Rutgers University, however it is 
 * based on nslookup and other pieces of named tools, so it needs
 * that copyright notice.
 */

/*
 * Extensively modified by E. Wassenaar, Nikhef-H, <e07@nikhef.nl>
 *
 * - Major overhaul of the whole code.
 * - Zone listing section completely rewritten.
 * - It is now possible to do recursive listings into subdomains.
 * - Maintain resource record statistics during zone listings.
 * - Very rigid error checking, with more verbose error messages.
 * - Exploit multiple server addresses if available.
 * - Option to exploit only primary server for zone transfers.
 * - Option to exclude info from names that do not reside in a domain.
 * - Implement timeout handling during connect and read.
 * - Write resource record output to optional logfile.
 * - Special MB tracing by recursively expanding MR and MG records.
 * - Special mode to check SOA records at each nameserver for domain.
 * - Special mode to check inverse mappings of host addresses.
 * - Code is extensively documented.
 */

#ifndef lint
static char Version[] = "@(#)host.c	e07@nikhef.nl (Eric Wassenaar) 910415";
#endif lint

/*
 * This program must be linked explicitly with the resolver library
 * if the default gethostbyname() or gethostbyaddr() functions use
 * a non-standard strategy (e.g. on Gould). The functions in the
 * resolver library call on the nameserver, and fall back on the
 * hosts file only if no nameserver is running (ECONNREFUSED).
 *
 * The version of the resolver should be BIND 4.8.2 or later.
 * The crucial include files are <netdb.h>, (resolv.h> and
 * <arpa/nameser.h>. These files are assumed to be present in
 * the /usr/include directory.
 *
 * The resolver source code depends on the definition of the BSD
 * pre-processor variable. This variable is usually defined in
 * the file <sys/param.h>.
 *
 * The hostent struct defined in <netdb.h> is assumed to handle
 * multiple addresses in h_addr_list[]. Usually this is true if
 * BSD >= 43.
 *
 * For smooth porting to both BSD and SYSV environments:
 * - Do not use the function value returned by sprintf().
 *   It is of different type in the two environments.
 * - Use <string.h> instead of <strings.h>.
 *
 * The treatment of TXT records has changed from 4.8.2 to 4.8.3.
 * Formerly, the data consisted simply of the text string.
 * Now, the text string is preceded by the character count with
 * a maximum of 255, and multiple strings are embedded if the
 * total character count exceeds 255.
 * We handle only the new situation in this program, assuming
 * that nobody uses TXT records before 4.8.3.
 *
 * Note that in 4.8.3 PACKETSZ from nameser.h is still at 512,
 * whereas MAXDATA from db.h has increased from 256 to 2048.
 * The nameserver reads queries in a buffer of size BUFSIZ.
 *
 * The gethostbyname() routine in 4.8.3 interprets dotted quads
 * (if not terminated with a dot) and simulates a gethostbyaddr(),
 * but we will not rely on it, and handle dotted quads ourselves.
 *
 * On some systems a bug in _doprnt() exists which prevents
 * printf("%.*s", n, string) to be printed correctly if n == 0.
 */

/*
 * Usage:	host [options] name [server]
 *
 * Regular command line options:
 * ----------------------------
 *
 * -t type	specify query type; default is T_A for normal mode
 *
 * -a		specify query type T_ANY
 *
 * -v		print verbose messages (-vv is very verbose)
 *
 * -d		print debugging output (-dd prints even more)
 *
 * Special mode options.
 * --------------------
 *
 * -l		special mode to generate zone listing for domain
 *
 * -C		special mode to check SOA records for domain
 *
 * -H		special mode to count hosts residing in domain
 *
 * -L level	do recursive domain listing/checking this levels deep
 *
 * -p		use primary nameserver of domain for zone transfers
 *
 * -S		print zone resource record statistics
 *
 * -A		special mode to check reverse mappings of host addresses
 *
 * Miscellaneous options.
 * ---------------------
 *
 * -T		print ttl value during non-verbose output
 *
 * -e		exclude info from names that do not reside in domain
 *
 * -f file	log resource record output also in given file
 *
 * -i		generate inverse in-addr.arpa query for dotted quad
 *
 * -m		specify query type T_MAILB and trace MB records
 *
 * -q		be quiet about some non-fatal errors
 *
 * Seldom used options.
 * -------------------
 *
 * -c class	specify query class; default is C_IN
 *
 * -r		do not use recursion when querying nameserver
 *
 * -s secs	specify timeout value in seconds; default is 2 * 5
 *
 * -w		wait until nameserver becomes available
 *
 */


#include <stdio.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/time.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <signal.h>
#include <sysexits.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/nameser.h>
#include <resolv.h>
#include <netdb.h>
#include <setjmp.h>

#ifndef T_TXT
#define T_TXT	16
#endif
#ifndef C_HS
#define C_HS	4
#endif
#ifndef NO_DATA
#define NO_DATA	NO_ADDRESS
#endif

#define T_NONE	0		/* yet unspecified resource record type */
#define T_FIRST	T_A		/* first possible type in resource record */
#define T_LAST	T_AXFR - 1	/* last  possible type in resource record */

#define TRUE	1
#define FALSE	0

#define NOT_DOTTED_QUAD ((u_long)-1)

#define bitset(a,b) (((a) & (b)) != 0)
#define sameword(a,b) (strcasecmp(a,b) == 0)
#define samepart(a,b) (strncasecmp(a,b,strlen(b)) == 0)
#define newstr(a) strcpy(xalloc(strlen(a) + 1), a)

#if PACKETSZ > 1024
#define MAXPACKET PACKETSZ
#else
#define MAXPACKET 1024
#endif

typedef union {
	HEADER header;
	u_char packet[MAXPACKET];
} querybuf;

#ifdef lint
#define EXTERN
#else
#define EXTERN extern
#endif lint

extern int errno;
EXTERN int h_errno;		/* defined in gethostnamadr.c */
EXTERN struct state _res;	/* defined in res_init.c */

char cnamebuf[MAXDNAME+1];
char *cname = NULL;		/* name to which CNAME is aliased */

char mnamebuf[MAXDNAME+1];
char *mname = NULL;		/* name to which MR or MG is aliased */

char soanamebuf[MAXDNAME+1];
char *soaname = NULL;		/* domain name of SOA record */

char subnamebuf[MAXDNAME+1];
char *subname = NULL;		/* domain name of NS record */

char adrnamebuf[MAXDNAME+1];
char *adrname = NULL;		/* domain name of A record */

char servername[MAXDNAME+1];
char *server = NULL;		/* name of explicit server to query */

char *logfilename = NULL;	/* name of log file */
FILE *logfile = NULL;		/* default is stdout only */

char *queryname = NULL;		/* the name about which to query */
int querytype = T_NONE;		/* the type of the query */
int queryclass = C_IN;		/* the class of the query */

int debug = 0;			/* print resolver debugging output */
int quiet = FALSE;		/* suppress some warning messages */
int verbose = 0;		/* verbose mode for extra output */
int inverse = FALSE;		/* generate inverse in-addr.arpa queries */
int primary = FALSE;		/* use primary server for zone transfers */
int ttlprint = FALSE;		/* print ttl value in non-verbose mode */
int waitmode = FALSE;		/* wait until server becomes available */
int mailmode = FALSE;		/* trace MG and MR into MB records */
int addrmode = FALSE;		/* check reverse mappings of addresses */
int listmode = FALSE;		/* generate zone listing of domain */
int hostmode = FALSE;		/* count real hosts residing within domain */
int checkmode = FALSE;		/* check SOA records at each nameserver */
int recursive = 0;		/* recursive listmode maximum level */
int exclusive = FALSE;		/* exclude records that are not in domain */
int statistics = FALSE;		/* print resource record statistics */

extern u_long inet_addr();
extern char *inet_ntoa();
extern char *strcpy();
extern char *index();

u_char *skip_qr();
u_char *next_rr();
char *pr_type();
char *pr_class();
char *in_addr_arpa();
char *decode_error();
char *get_primary();
char *xalloc();

#ifdef DEBUG
#define ASSERT(condition)\
{\
	if (!(condition))\
	{\
		fprintf(stderr, "assertion botch: ");\
		fprintf(stderr, "%s(%d): ", __FILE__, __LINE__);\
		fprintf(stderr, "%s\n", "condition");\
	}\
}
#else
#define ASSERT(condition)
#endif DEBUG

char Usage[] =
"\
Usage: host [-v] [-a] [-t querytype]  name  [server]\n\
       host [-v] [-a] [-t querytype] [-e] [-p] [-S]  -l domain  [server]\n\
       host [-v] -H domain\n\
       host [-v] -C domain\n\
       host [-v] -A host\n\
Other: [-T] [-d] [-i] [-m] [-q] [-f logfile] [-s secs] [-c class] [-r] [-w]\n\
";


/*
** MAIN -- Start of program host
** -----------------------------
**
**	Exits:
**		EX_OK		Operation successfully completed
**		EX_UNAVAILABLE	Could not obtain requested information
**		EX_CANTCREAT	Cannot create specified logfile
**		EX_USAGE	Improper parameter/option specified
**		EX_OSERR	Cannot obtain resources
*/

main(argc, argv)
int argc;
char *argv[];
{
	register int i;
	register char *option;
	struct hostent *hp;
	char newnamebuf[MAXDNAME+1];
	char *newname = NULL;		/* name to which CNAME is aliased */
	int ncnames = 0;		/* count of CNAMEs in chain */
	u_long addr;			/* explicit address of query */
	u_long svaddr;			/* explicit address of server */
	int result;			/* result status of action taken */

/*
 * Synchronize stdout and stderr in case output is redirected.
 * Note that some SYSV implementations may have setlinebuf().
 */
#if defined(SYSV)
	setvbuf(stdout, NULL, _IOLBF, BUFSIZ);
#else
	setlinebuf(stdout);
#endif

/*
 * Initialize resolver. See print_res() for details.
 * Pickup current values and set new defaults.
 * Query for optional server is also done with new defaults.
 * Old defaults are (RES_RECURSE | RES_DEFNAMES | RES_DNSRCH)
 */
	res_init();

	/* we handle default domains ourselves, thank you */
	_res.options |=  RES_DEFNAMES;
	_res.options &= ~RES_DNSRCH;

	_res.retry = 2;		/* number  of retries, default = 4 */
	_res.retrans = 5;	/* timeout in seconds, default = 4 or 5 */

/*
 * Scan command line options and flags.
 */
	while (argc > 2 && argv[1][0] == '-')
	{
	    for (option = &argv[1][1]; *option; option++)
		switch (*option)
		{
		    case 'w' :
			waitmode = TRUE;
			_res.retry = 1;
			_res.retrans = 10;
			break;

		    case 's' :
			argv++; argc--;
			_res.retry = 1;
			_res.retrans = atoi(argv[1]);
			if (_res.retrans <= 0)
				fatal("Invalid timeout value %s", argv[1]);
			break;

		    case 'r' :
			_res.options &= ~RES_RECURSE;
			break;

		    case 'd' :
			debug++;		/* increment debugging level */
			_res.options |= RES_DEBUG;
			break;

		    case 'v' :
			verbose++;		/* increment verbosity level */
			break;

		    case 'q' :
			quiet = TRUE;
			break;

		    case 'i' :
			inverse = TRUE;
			break;

		    case 'p' :
			primary = TRUE;
			break;

		    case 'e' :
			exclusive = TRUE;
			break;

		    case 'A' :
			addrmode = TRUE;
			break;

		    case 'S' :
			statistics = TRUE;
			break;

		    case 'T' :
			ttlprint = TRUE;
			break;

		    case 'H' :
			hostmode = TRUE;
			listmode = TRUE;
			if (querytype == T_NONE)
				querytype = -1;	/* suppress zone data output */
			break;

		    case 'C' :
			checkmode = TRUE;
			listmode = TRUE;
			if (querytype == T_NONE)
				querytype = -1;	/* suppress zone data output */
			break;

		    case 'l' :
			listmode = TRUE;
			break;

		    case 'L' :
			argv++; argc--;
			recursive = atoi(argv[1]);
			if (recursive <= 0)
				fatal("Invalid recursion level %s", argv[1]);
			break;

		    case 'f' :
			argv++; argc--;
			logfilename = argv[1];
			break;

		    case 'c' :
			argv++; argc--;
			queryclass = parse_class(argv[1]);
			if (queryclass < 0)
				fatal("Invalid query class %s", argv[1]);
			break;

		    case 't' :
			argv++; argc--;
			querytype = parse_type(argv[1]);
			if (querytype < 0)
				fatal("Invalid query type %s", argv[1]);
			break;

		    case 'a' :
			querytype = T_ANY;	/* filter anything available */
			break;

		    case 'm' :
			mailmode = TRUE;
			querytype = T_MAILB;	/* filter MINFO/MG/MR/MB data */
			break;

		    default:
			fprintf(stderr, Usage);
			exit(EX_USAGE);
		}

		argv++; argc--;
	}

	if (argc < 2 || argv[1][0] == '-')
	{
		fprintf(stderr, Usage);
		exit(EX_USAGE);
	}

/*
 * Analyze name and type to be queried about.
 * In regular mode, the querytype is used to formulate the nameserver
 * query, and any response is filtered out when processing the answer.
 * In listmode, the querytype is used to filter out the proper records.
 */
	queryname = argv[1];
	if (queryname[0] == '\0')
		queryname = ".";

	if (sameword(queryname, "."))
		addr = NOT_DOTTED_QUAD;
	else
		addr = inet_addr(queryname);

	if ((addr != NOT_DOTTED_QUAD) && inverse && !addrmode)
	{
		queryname = in_addr_arpa(queryname);
		if (queryname == NULL)
			fatal("Invalid dotted quad %s", argv[1]);

		addr = NOT_DOTTED_QUAD;

		if ((querytype == T_NONE) && !listmode)
			querytype = T_PTR;
	}

	if ((querytype == T_NONE) && !listmode)
	{
		if (addr == NOT_DOTTED_QUAD)
			querytype = T_A;
		else
			querytype = T_PTR;
	}

	if (listmode && (addr != NOT_DOTTED_QUAD))
		fatal("Invalid query name %s", argv[1]);

	if (verbose)
		print_types(queryname, querytype);

/*
 * Check for possible alternative server.
 */
	if (argc > 2)
	{
		server = argv[2];
		svaddr = inet_addr(server);
		if (svaddr == NOT_DOTTED_QUAD)
		{
			hp = gethostbyname(server);
			if (hp == NULL)
			{
				fprintf(stderr, "Error in looking up server name:\n");
				hp_error(h_errno, server, T_A);
				exit(EX_UNAVAILABLE);
			}

			for (i = 0; i < MAXNS && hp->h_addr_list[i]; i++)
			{
				_res.nsaddr_list[i].sin_family = AF_INET;
				_res.nsaddr_list[i].sin_port = htons(NAMESERVER_PORT);
				_res.nsaddr_list[i].sin_addr = *(struct in_addr *)hp->h_addr_list[i];
			}
			_res.nscount = i;

			server = strcpy(servername, hp->h_name);
			print_host("Server", hp);
		}
		else
		{
			_res.nsaddr.sin_family = AF_INET;
			_res.nsaddr.sin_port = htons(NAMESERVER_PORT);
			_res.nsaddr.sin_addr.s_addr = svaddr;
			_res.nscount = 1;

			hp = gethostbyaddr((char *)&svaddr, sizeof(svaddr), AF_INET);
			if (hp != NULL)
			{
				server = strcpy(servername, hp->h_name);
				print_host("Server", hp);
			}
			else
			{
				server = strcpy(servername, inet_ntoa(_res.nsaddr.sin_addr));
				printf("Server: %s\n\n", server);
			}
		}
	}

	if (debug > 1 || verbose > 1)
		print_res();

/*
 * Open log file if requested.
 */
	if (logfilename)
	{
		logfile = fopen(logfilename, "w");
		if (logfile == NULL)
		{
			perror(logfilename);
			exit(EX_CANTCREAT);
		}
	}

/*
 * Special mode to check inverse mappings of host addresses.
 */
	if (addrmode)
	{
		if (addr == NOT_DOTTED_QUAD)
			result = check_addr(queryname);
		else
			result = check_name(addr);

		if (result == FALSE)
			exit(EX_UNAVAILABLE);
		exit(EX_OK);
	}

/*
 * Special mode to list contents of specified domain.
 */
	if (listmode)
	{
		result = list_domain(queryname);

		if (result == FALSE)
			exit(EX_UNAVAILABLE);
		exit(EX_OK);
	}

/*
 * Regular mode to query about specified host.
 */
	result = FALSE;
	h_errno = TRY_AGAIN;

	while (result == FALSE && h_errno == TRY_AGAIN)
	{
		if (addr == NOT_DOTTED_QUAD)
		{
			if (newname == NULL)
				result = get_hostinfo(queryname);
			else
				result = get_hostinfo(newname);

			if (cname)
			{
				ncnames++;
				if (ncnames > 5)
				{
					fprintf(stderr, "Too many CNAMEs. Possible loop.\n");
					exit(EX_UNAVAILABLE);
				}

				newname = strcpy(newnamebuf, cname);
				cname = NULL;

				result = FALSE;
				h_errno = TRY_AGAIN;
				continue;
			}
		}
		else
		{
			hp = gethostbyaddr((char *)&addr, sizeof(addr), AF_INET);
			if (hp != NULL)
			{
				print_host("Name", hp);
				result = TRUE;
			}
		}

		if (!waitmode)
			break;
	}

	if (result == FALSE)
	{
		hp_error(h_errno, queryname, querytype);
		exit(EX_UNAVAILABLE);
	}
	exit(EX_OK);
}


/*
** FATAL -- Abort program when illegal option encountered
** ------------------------------------------------------
*/

/*VARARGS1*/
fatal(fmt, a, b, c, d)
char *fmt;
{
	fprintf(stderr, fmt, a, b, c, d);
	fprintf(stderr, "\n");
	exit(EX_USAGE);
}


/*
** PARSE_TYPE -- Decode rr type from input string
** ----------------------------------------------
**
**	Returns:
**		Value of resource record type.
**		-1 if specified record name is invalid.
**
**	Note.	T_MD, T_MF, T_MAILA are obsolete, but recognized.
**		T_AXFR is not allowed to be specified as query type.
*/

parse_type(string)
char *string;
{
	int type;

	if (sameword(string, "A"))	return(T_A);
	if (sameword(string, "NS"))	return(T_NS);
	if (sameword(string, "MD"))	return(T_MD);		/* obsolete */
	if (sameword(string, "MF"))	return(T_MF);		/* obsolete */
	if (sameword(string, "CNAME"))	return(T_CNAME);
	if (sameword(string, "SOA"))	return(T_SOA);
	if (sameword(string, "MB"))	return(T_MB);
	if (sameword(string, "MG"))	return(T_MG);
	if (sameword(string, "MR"))	return(T_MR);
	if (sameword(string, "NULL"))	return(T_NULL);
	if (sameword(string, "WKS"))	return(T_WKS);
	if (sameword(string, "PTR"))	return(T_PTR);
	if (sameword(string, "HINFO"))	return(T_HINFO);
	if (sameword(string, "MINFO"))	return(T_MINFO);
	if (sameword(string, "MX"))	return(T_MX);
	if (sameword(string, "TXT"))	return(T_TXT);

	if (sameword(string, "UINFO"))	return(T_UINFO);
	if (sameword(string, "UID"))	return(T_UID);
	if (sameword(string, "GID"))	return(T_GID);
	if (sameword(string, "UNSPEC"))	return(T_UNSPEC);

	if (sameword(string, "AXFR"))	return(-1);		/* illegal */
	if (sameword(string, "MAILB"))	return(T_MAILB);
	if (sameword(string, "MAILA"))	return(T_MAILA);	/* obsolete */
	if (sameword(string, "ANY"))	return(T_ANY);
	if (sameword(string, "*"))	return(T_ANY);

	type = atoi(string);
	if (type >= T_FIRST && type <= T_LAST)
		return(type);

	return(-1);
}


/*
** PARSE_CLASS -- Decode rr class from input string
** ------------------------------------------------
**
**	Returns:
**		Value of resource class.
**		-1 if specified class name is invalid.
*/

parse_class(string)
char *string;
{
	int class;

	if (sameword(string, "IN"))	return(C_IN);
	if (sameword(string, "CHAOS"))	return(C_CHAOS);
	if (sameword(string, "HS"))	return(C_HS);

	if (sameword(string, "ANY"))	return(C_ANY);
	if (sameword(string, "*"))	return(C_ANY);

	class = atoi(string);
	if (class > 0)
		return(class);

	return(-1);
}


/*
** IN_ADDR_ARPA -- Convert dotted quad string to inverse in-addr.arpa
** ------------------------------------------------------------------
**
**	Returns:
**		Pointer to inverse in-addr.arpa domain name.
**		NULL in case of invalid dotted quad input string.
*/

char *
in_addr_arpa(dottedquad)
char *dottedquad;
{
	static char addrbuf[32];
	unsigned int a[4];
	int n;

	n = sscanf(dottedquad, "%u.%u.%u.%u", &a[0], &a[1], &a[2], &a[3]);
	switch (n)
	{
	    case 4:
		(void) sprintf(addrbuf, "%u.%u.%u.%u.in-addr.arpa.",
			a[3]&0xff, a[2]&0xff, a[1]&0xff, a[0]&0xff);
		break;

	    case 3:
		(void) sprintf(addrbuf, "%u.%u.%u.in-addr.arpa.",
			a[2]&0xff, a[1]&0xff, a[0]&0xff);
		break;

	    case 2:
		(void) sprintf(addrbuf, "%u.%u.in-addr.arpa.",
			a[1]&0xff, a[0]&0xff);
		break;

	    case 1:
		(void) sprintf(addrbuf, "%u.in-addr.arpa.",
			a[0]&0xff);
		break;

	    default:
		return(NULL);
	}

	while (--n >= 0)
		if (a[n] > 255)
			return(NULL);

	return(addrbuf);
}


/*
** PRINT_HOST -- Print hostname and address of hostent struct
** ----------------------------------------------------------
**
**	Returns:
**		None.
*/

print_host(heading, hp)
char *heading;
struct hostent *hp;
{
	register char **ap;

	printf("%s: %s", heading, hp->h_name);

	for (ap = hp->h_addr_list; ap && *ap; ap++)
	{
		if (ap == hp->h_addr_list)
			printf("\nAddress:");
		printf(" %s", inet_ntoa(*(struct in_addr *)*ap));
	}

	for (ap = hp->h_aliases; ap && *ap && **ap; ap++)
	{
		if (ap == hp->h_aliases)
			printf("\nAliases:");
		printf(" %s", *ap);
	}

	printf("\n\n");
}


/*
** PRINT_RES -- Print resolver database information
** ------------------------------------------------
**
**	Returns:
**		None.
*/

print_res()
{
	register int i;
	register char **domain;

/*
 * The default domain is defined by the "domain" entry in /etc/resolv.conf
 * if not overridden by the environment variable "LOCALDOMAIN".
 * If still not defined, gethostname() may yield a fully qualified hostname.
 */
	printf("Default domain:");
	if (_res.defdname[0] != '\0')
		printf(" %s", _res.defdname);
	printf("\n");

/*
 * The search domains are extracted from the default domain components,
 * but may be overridden by "search" directives in /etc/resolv.conf
 * since 4.8.3.
 */
	printf("Search domains:");
	for (domain = _res.dnsrch; *domain; domain++)
		printf(" %s", *domain);
	printf("\n");

/*
 * Each try by res_send() to contact a nameserver will timeout after
 * _res.retrans seconds. It will do _res.retry attempts for to contact
 * each of the _res.nscount nameserver addresses before giving up.
 */
	printf("Timeout per retry: %d secs\n", _res.retrans);
	printf("Number of retries: %d\n", _res.retry);

	printf("Number of addresses: %d\n", _res.nscount);
	for (i = 0; i < _res.nscount; i++)
		printf("%s\n", inet_ntoa(_res.nsaddr_list[i].sin_addr));

/*
 * The resolver options are initialized by res_init() to contain the
 * defaults settings (RES_RECURSE | RES_DEFNAMES | RES_DNSRCH)
 * The various options have the following meaning:
 *
 *	RES_INIT	set after res_init() has been called
 *	RES_DEBUG	let the resolver modules print debugging info
 *	RES_AAONLY	want authoritative answers only (not implemented)
 *	RES_USEVC	use tcp virtual circuit instead of datagrams
 *	RES_PRIMARY	use primary nameserver only (not implemented)
 *	RES_IGNTC	ignore datagram truncation; don't switch to tcp
 *	RES_RECURSE	forward query if answer not locally available
 *	RES_DEFNAMES	add default domain to queryname without dot
 *	RES_STAYOPEN	keep tcp socket open for subsequent queries
 *	RES_DNSRCH	append search domains even to queryname with dot
 */
	printf("Options set:");
	if (bitset(RES_INIT,      _res.options)) printf(" INIT");
	if (bitset(RES_DEBUG,     _res.options)) printf(" DEBUG");
	if (bitset(RES_AAONLY,    _res.options)) printf(" AAONLY");
	if (bitset(RES_USEVC,     _res.options)) printf(" USEVC");
	if (bitset(RES_PRIMARY,   _res.options)) printf(" PRIMARY");
	if (bitset(RES_IGNTC,     _res.options)) printf(" IGNTC");
	if (bitset(RES_RECURSE,   _res.options)) printf(" RECURSE");
	if (bitset(RES_DEFNAMES,  _res.options)) printf(" DEFNAMES");
	if (bitset(RES_STAYOPEN,  _res.options)) printf(" STAYOPEN");
	if (bitset(RES_DNSRCH,    _res.options)) printf(" DNSRCH");
	printf("\n");

	printf("Options clr:");
	if (!bitset(RES_INIT,     _res.options)) printf(" INIT");
	if (!bitset(RES_DEBUG,    _res.options)) printf(" DEBUG");
	if (!bitset(RES_AAONLY,   _res.options)) printf(" AAONLY");
	if (!bitset(RES_USEVC,    _res.options)) printf(" USEVC");
	if (!bitset(RES_PRIMARY,  _res.options)) printf(" PRIMARY");
	if (!bitset(RES_IGNTC,    _res.options)) printf(" IGNTC");
	if (!bitset(RES_RECURSE,  _res.options)) printf(" RECURSE");
	if (!bitset(RES_DEFNAMES, _res.options)) printf(" DEFNAMES");
	if (!bitset(RES_STAYOPEN, _res.options)) printf(" STAYOPEN");
	if (!bitset(RES_DNSRCH,   _res.options)) printf(" DNSRCH");
	printf("\n");

	printf("\n");
}


int stats[T_ANY+1];		/* count of resource records per type */

#define clear_statistics() bzero((char *)stats, sizeof(stats))


/*
** PRINT_STATISTICS -- Print resource record statistics
** ----------------------------------------------------
**
**	Returns:
**		None.
**
**	Inputs:
**		The stats[] counts have been updated by next_rr().
*/

print_statistics(name, filter)
char *name;				/* name of domain we are listing */
int filter;				/* type of records we want to see */
{
	register int type;
	int nrecords;

	for (type = T_FIRST; type <= T_LAST; type++)
	{
		nrecords = stats[type];
		if (nrecords > 0 || (filter != T_ANY && want_rr(type, filter)))
		{
			printf("Found %4d %-5.5s record%s within %s\n",
				nrecords, pr_type(type),
				nrecords == 1 ? " " : "s", name);
		}
	}
}


/*
** PRINT_TYPES -- Print resource record types wanted
** -------------------------------------------------
**
**	Returns:
**		None.
*/

print_types(name, filter)
char *name;				/* name we want to query about */
int filter;				/* type of records we want to see */
{
	register int type;

	if (filter >= T_NONE)
	{
		printf("Query about %s for record types", name);
		if (filter == T_ANY)
			printf(" %s", pr_type(T_ANY));
		else
			for (type = T_FIRST; type <= T_LAST; type++)
				if (want_rr(type, filter))
					printf(" %s", pr_type(type));
		printf("\n");
	}
}


/*
** HP_ERROR -- Print error message from errno and h_errno
** ------------------------------------------------------
**
**	Returns:
**		None.
**
**	Inputs:
**		The global variable server, if set, contains
**		the name of the server that was contacted.
*/

hp_error(error, name, type) 
int error;				/* error code from h_errno */
char *name;				/* full name we queried about */
int type;				/* record type we queried about */
{
	switch (errno)
	{
	    case ECONNREFUSED:
		/*
		 * The contacted host does not have a nameserver running.
		 */
		if (server)
			fprintf(stderr, "Nameserver %s not running\n", server);
		else
			fprintf(stderr, "Nameserver not running\n");
		break;

	    case ETIMEDOUT:
		/*
		 * The contacted host did not give any reply at all.
		 */
		if (server)
			fprintf(stderr, "Nameserver %s not responding\n", server);
		else
			fprintf(stderr, "Nameserver not responding\n");
		break;
	}

	switch (error)
	{
	    case HOST_NOT_FOUND:
		/*
		 * The specified name does not exist at all.
		 */
		fprintf(stderr,"%s not found (Authoritative answer)\n", name);
		break;

	    case TRY_AGAIN:
		/*
		 * Some intermediate server failure.
		 */
		fprintf(stderr,"%s %s record not found, try again\n", name, pr_type(type));
		break;

	    case NO_RECOVERY:
		/*
		 * Some irrecoverable format error, or server refusal.
		 */
		fprintf(stderr,"%s %s record not found, no recovery\n", name, pr_type(type));
		break;

	    case NO_ADDRESS:
		/*
		 * The name exists, but the specified type does not exist.
		 */
		fprintf(stderr,"%s has no %s record\n", name, pr_type(type));
		break;

	    default:
		/*
		 * Unknown cause for failure.
		 */
		fprintf(stderr,"%s %s record not found\n", name, pr_type(type));
		break;
	}
}


/*
** DECODE_ERROR -- Convert nameserver error code to error message
** --------------------------------------------------------------
**
**	Returns:
**		Pointer to appropriate error message.
*/

char *
decode_error(error)
int error;				/* error code from bp->rcode */
{
	switch (error)
	{
	    case NOERROR: 	return("no error");
	    case FORMERR:	return("format error");
	    case SERVFAIL:	return("server failed");
	    case NXDOMAIN:	return("non-existent domain");
	    case NOTIMP:	return("not implemented");
	    case REFUSED:	return("query refused");
	    case NOCHANGE:	return("no change");
	}

	return("unknown error");
}


/*
** PRINT_STATUS -- Print result status after nameserver query
** ----------------------------------------------------------
**
**	Returns:
**		None.
**
**	Conditions:
**		The size of the answer buffer must have been
**		checked before to be of sufficient length,
**		i.e. to contain at least the buffer header.
*/

print_status(answerbuf)
querybuf *answerbuf;			/* address of answer buffer */
{
	HEADER *bp;
	int ancount;
	int failed;

	bp = (HEADER *)answerbuf;
	ancount = ntohs(bp->ancount);
	failed = (bp->rcode != NOERROR || ancount == 0);

	printf("Query %s, %d answer%s, %sstatus: %s\n",
		failed ? "failed" : "done",
		ancount, ancount == 1 ? "" : "s",
		bp->aa ? "authoritative " : "",
		decode_error((int)bp->rcode));
}


/*
** GET_HOSTINFO -- Principal routine to query about given name
** -----------------------------------------------------------
**
**	Returns:
**		TRUE if requested info was obtained successfully.
**		FALSE otherwise.
*/

get_hostinfo(name)
char *name;				/* name to query about */
{
	extern char *hostalias();
	register char **domain;
	register char *cp;
	int dot;			/* number of dots in query name */
	int result;			/* result status of action taken */

/*
 * Single dot means root domain.
 */
	if (sameword(name, "."))
	{
		result = get_domaininfo(name, (char *)NULL);
		return(result);
	}

/*
 * Count number of dots.
 */
	for (dot = 0, cp = name; *cp; cp++)
		if (*cp == '.')
			dot++;

/*
 * Check for aliases of single name.
 */
	if (dot == 0 && (cp = hostalias(name)) != NULL)
	{
		if (verbose)
			printf("Aliased to \"%s\"\n", cp);
		result = get_domaininfo(cp, (char *)NULL);
		return(result);
	}

/*
 * Trailing dot means absolute address.
 */
	if (dot != 0 && cp[-1] == '.')
	{
		cp[-1] = 0;
		result = get_domaininfo(name, (char *)NULL);
		cp[-1] = '.';
		return(result);
	}

/*
 * Append own domain if appropriate.
 */
	if ((dot == 0 && bitset(RES_DEFNAMES, _res.options)) ||
	    (dot != 0 && bitset(RES_DNSRCH, _res.options)))
	{
		for (domain = _res.dnsrch; *domain; domain++)
		{
			result = get_domaininfo(name, *domain);
			if (result)
				return(result);

			/* in case nameserver not present */
			if (errno == ECONNREFUSED)
				return(FALSE);
		}
	}

/*
 * Single host name lookup failed.
 */
	if (dot == 0)
	{
		/* set status in case we never queried */
		if (!bitset(RES_DEFNAMES, _res.options))
			h_errno = HOST_NOT_FOUND;
		return(FALSE);
	}

/*
 * Rest means fully qualified.
 */
	result = get_domaininfo(name, (char *)NULL);
	return(result);
}


/*
** GET_DOMAININFO -- Fetch and print desired info about name in domain
** -------------------------------------------------------------------
**
**	Returns:
**		TRUE if requested info was obtained successfully.
**		FALSE otherwise.
*/

get_domaininfo(name, domain)
char *name;				/* name to query about */
char *domain;				/* domain to which name is relative */
{
	char namebuf[2*MAXDNAME+2];	/* buffer to store full domain name */
	querybuf answer;
	int anslen;
	int result;

	if (verbose)
	{
		if (domain == NULL || domain[0] == '\0')
			printf("Trying %s ...\n", name);
		else
			printf("Trying %s within %s ...\n", name, domain);
	}

/*
 * Construct the actual domain name.
 * A null domain means the given name is already fully qualified.
 */
	if (domain == NULL || domain[0] == '\0')
		(void) sprintf(namebuf, "%.*s", MAXDNAME, name);
	else
		(void) sprintf(namebuf, "%.*s.%.*s", MAXDNAME, name, MAXDNAME, domain);
	name = namebuf;

/*
 * Fetch the desired info, and print any relevant data.
 */
	anslen = get_info(&answer, name, querytype, queryclass);
	if (anslen < 0)
		return(FALSE);

	result = print_info(&answer, anslen, name, T_ANY);
	return(result == NOERROR ? TRUE : FALSE);
}


/*
** GET_INFO -- Basic routine to issue a nameserver query
** -----------------------------------------------------
**
**	Returns:
**		Length of nameserver answer buffer, if obtained.
**		-1 if an error occurred (h_errno is set appropriately).
*/

get_info(answerbuf, name, type, class)
querybuf *answerbuf;			/* address of buffer to store answer */
char *name;				/* full name to query about */
int type;				/* specific resource record type */
int class;				/* specific resource record class */
{
	querybuf query;
	HEADER *bp;
	int ancount;
	int n;

/*
 * Construct query, and send it to the nameserver.
 * res_send() will fail if no nameserver responded. In this case
 * the possible values for errno are ECONNREFUSED and ETIMEDOUT.
 * If we got an answer, errno should be reset. Note that res_send()
 * may have left an errno in case it has used datagrams.
 */
	errno = 0;	/* reset before querying nameserver */

	n = res_mkquery(QUERY, name, class, type,
			(char *)NULL, 0, (char *)NULL,
			(char *)&query, sizeof(querybuf));
	if (n < 0)
	{
		if (debug)
			fprintf(stderr, "res_mkquery failed\n");
		h_errno = NO_RECOVERY;
		return(-1);
	}

	n = res_send((char *)&query, n, (char *)answerbuf, sizeof(querybuf));
	if (n < 0)
	{
		if (debug)
			fprintf(stderr, "res_send failed\n");
		h_errno = TRY_AGAIN;
		return(-1);
	}

	errno = 0;	/* reset after we got an answer */

	if (n < sizeof(HEADER))
	{
		pr_error("answer length %d too short", n);
		h_errno = NO_RECOVERY;
		return(-1);
	}

/*
 * Analyze the status of the answer from the nameserver.
 */
	if (debug || verbose)
		print_status(answerbuf);

	bp = (HEADER *)answerbuf;
	ancount = ntohs(bp->ancount);

	if (bp->rcode != NOERROR || ancount == 0)
	{
		switch (bp->rcode)
		{
		    case NXDOMAIN:
			/* check if it's an authoritative answer */
			if (bp->aa)
				h_errno = HOST_NOT_FOUND;
			else
				h_errno = TRY_AGAIN;
			break;

		    case NOERROR:
			h_errno = NO_DATA;
			break;

		    case SERVFAIL:
			h_errno = TRY_AGAIN;
			break;

		    case FORMERR:
		    case NOTIMP:
		    case REFUSED:
		    case NOCHANGE:
			h_errno = NO_RECOVERY;
			break;

		    default:
			h_errno = NO_RECOVERY;
			break;
		}
		return(-1);
	}

	h_errno = 0;
	return(n);
}


/*
** PRINT_INFO -- Check resource records in answer and print relevant data
** ----------------------------------------------------------------------
**
**	Returns:
**		NOERROR if answer buffer was processed successfully.
**		FORMERR otherwise.
**
**	Side effects:
**		Will recurse on MAILB records if appropriate.
**		See also side effects of the next_rr() routine.
*/

print_info(answerbuf, answerlen, name, filter)
querybuf *answerbuf;			/* address of answer buffer */
int answerlen;				/* length of answer buffer */
char *name;				/* full name we are querying about */
int filter;				/* type of records we want to see */
{
	HEADER *bp;
	int qdcount, ancount, nscount, arcount;
	u_char *msg, *eom, *cp;

	bp = (HEADER *)answerbuf;
	qdcount = ntohs(bp->qdcount);
	ancount = ntohs(bp->ancount);
	nscount = ntohs(bp->nscount);
	arcount = ntohs(bp->arcount);

	msg = (u_char *)answerbuf;
	eom = (u_char *)answerbuf + answerlen;
	cp  = (u_char *)answerbuf + sizeof(HEADER);

/*
 * Skip the query section in the response (present only in normal queries).
 */
	if (qdcount)
	{
		while (qdcount > 0 && cp < eom)
		{
			/* cp += dn_skipname(cp, eom) + QFIXEDSZ; */

			cp = skip_qr(name, cp, msg, eom);
			if (cp == NULL)
				return(FORMERR);
			qdcount--;
		}

		if (qdcount)
		{
			pr_error("invalid qdcount in response");
			return(FORMERR);
		}
	}

/*
 * Process the actual answer section in the response.
 * During zone transfers, this is the only section available.
 */
	if (ancount)
	{
		if (!listmode && verbose && !bp->aa)
			printf("The following answer is not authoritative:\n");

		while (ancount > 0 && cp < eom)
		{
			cp = next_rr(name, cp, msg, eom, filter);
			if (cp == NULL)
				return(FORMERR);
			ancount--;

		/*
		 * When we ask for address and there is a CNAME, it returns
		 * both the CNAME and the address.  Since we trace down the
		 * CNAME chain ourselves, we don't really want to print the
		 * address at this point.
		 */
			if (!listmode && !verbose && cname)
				return(NOERROR);

		/*
		 * Recursively expand MR or MG records into MB records.
		 */
			if (mailmode && !listmode && mname)
			{
				char newnamebuf[MAXDNAME+1];
				char *newname;

				newname = strcpy(newnamebuf, mname);
				mname = NULL;

				(void) get_hostinfo(newname);
			}
		}

		if (ancount)
		{
			pr_error("invalid ancount in response");
			return(FORMERR);
		}
	}

/*
 * The nameserver and additional info section are normally not processed.
 */
	if (!verbose || exclusive)
		return(NOERROR);

	if (nscount)
	{
		printf("Authoritative nameservers:\n");

		while (nscount > 0 && cp < eom)
		{
			cp = next_rr(name, cp, msg, eom, filter);
			if (cp == NULL)
				return(FORMERR);
			nscount--;
		}

		if (nscount)
		{
			pr_error("invalid nscount in response");
			return(FORMERR);
		}
	}

	if (arcount)
	{
		printf("Additional information:\n");

		while (arcount > 0 && cp < eom)
		{
			cp = next_rr(name, cp, msg, eom, filter);
			if (cp == NULL)
				return(FORMERR);
			arcount--;
		}

		if (arcount)
		{
			pr_error("invalid arcount in response");
			return(FORMERR);
		}
	}

	return(NOERROR);
}


/*
** PR_ERROR -- Print error message about encountered inconsistencies
** -----------------------------------------------------------------
**
**	Returns:
**		None.
*/

/*VARARGS1*/
pr_error(fmt, a, b, c, d)
char *fmt;
{
	fprintf(stderr, " *** ");
	fprintf(stderr, fmt, a, b, c, d);
	fprintf(stderr, "\n");
}


/*
** DOPRINT -- Output resource record data if this record is wanted
** ---------------------------------------------------------------
**
**	Returns:
**		None.
**
**	Inputs:
**		The global variable doprint is set by next_rr()
**		if we need to print the data.
*/

static int doprint;

/*VARARGS1*/
doprintf(fmt, a, b, c, d)
char *fmt;
{
	if (doprint)
	{
		printf(fmt, a, b, c, d);

		if (logfile)
			fprintf(logfile, fmt, a, b, c, d);
	}
}


/*
** WANT_RR -- Indicate whether the rr type matches the desired filter
** ------------------------------------------------------------------
**
**	Returns:
**		TRUE if the resource record type matches the filter.
**		FALSE otherwise.
*/

want_rr(type, filter)
int type;				/* resource record type */
int filter;				/* type of records we want to see */
{
	if (type == filter)
		return(TRUE);

	if (filter == T_ANY)
		return(TRUE);

	if (filter == T_NONE &&
	   (type == T_A || type == T_NS || type == T_PTR))
		return(TRUE);

	if (filter == T_MAILB &&
	   (type == T_MB || type == T_MR || type == T_MG || type == T_MINFO))
		return(TRUE);

	if (filter == T_MAILA &&
	   (type == T_MD || type == T_MF))
		return(TRUE);

	return(FALSE);
}


/*
** SAMEDOMAIN -- Check whether a name belongs to a domain
** ------------------------------------------------------
**
**	Returns:
**		TRUE if the given name lies anywhere in the domain, or
**		if the given name is the same as the domain and may be so.
**		FALSE otherwise.
*/

samedomain(name, domain, equal)
char *name;				/* the name under consideration */
char *domain;				/* the name of the domain */
int equal;				/* set if name may be same as domain */
{
	register char *dot;

	if (sameword(name, domain))
		return(equal);

	if (sameword(domain, "."))
		return(TRUE);

	dot = index(name, '.');
	while (dot != NULL)
	{
		if (sameword(dot+1, domain))
			return(TRUE);

		dot = index(dot+1, '.');
	}

	return(FALSE);
}


/*
** INDOMAIN -- Check whether a name belongs to a domain
** ----------------------------------------------------
**
**	Returns:
**		TRUE if the given name directly lies in the domain, or
**		if the given name is the same as the domain and may be so.
**		FALSE otherwise.
**
**	Kludge:
**		The special names "localhost" and "loopback" are
**		considered to be not part of a domain.
*/

indomain(name, domain, equal)
char *name;				/* the name under consideration */
char *domain;				/* the name of the domain */
int equal;				/* set if name may be same as domain */
{
	register char *dot;

	if (sameword(name, domain))
		return(equal);

	dot = index(name, '.');
	if (dot == NULL)
		return(sameword(domain, "."));

	if (!sameword(dot+1, domain))
		return(FALSE);

	if (samepart(name, "localhost") || samepart(name, "loopback"))
		return(FALSE);

	return(TRUE);
}


/*
** NEXT_RR -- Decode single resource record and output relevant data
** -----------------------------------------------------------------
**
**	Returns:
**		Pointer to position in answer buffer after current record.
**		NULL if there was a format error in the current record.
**
**	Outputs:
**		The global variable doprint is set appropriately
**		for use by doprintf().
**
**	Side effects:
**		Updates resource record statistics in stats[].
**		Sets soaname if this is an SOA record.
**		Sets subname if this is an NS record.
**		Sets adrname if this is an A record.
**		Sets cname if this is a valid CNAME record.
**		Sets mname if this is a valid MAILB record.
**		These variables must have been cleared before calling
**		print_info() and may be checked afterwards.
*/

u_char *
next_rr(name, cp, msg, eom, filter)
char *name;				/* full name we are querying about */
u_char *cp, *msg, *eom;			/* current positions in answer buf */
int filter;				/* type of records we want to see */
{
	char rname[MAXDNAME+1];		/* record name in LHS */
	char dname[MAXDNAME+1];		/* domain name in RHS */
	int type, class, ttl, dlen;	/* fixed values in every record */
	u_char *eor;			/* predicted position of next record */
	int n;
	struct in_addr inaddr;
	struct protoent *protocol;
	struct servent *service;

/*
 * Pickup the standard values present in each resource record.
 */
	n = expand(name, T_NONE, cp, msg, eom, rname);
	if (n < 0)
		return(NULL);
	cp += n;

	n = 3*sizeof(u_short) + sizeof(u_long);
	if (check_size(rname, T_NONE, cp, msg, eom, n) < 0)
		return(NULL);

	type = _getshort(cp);
	cp += sizeof(u_short);

	class = _getshort(cp);
	cp += sizeof(u_short);

	ttl = _getlong(cp);
	cp += sizeof(u_long);

	dlen = _getshort(cp);
	cp += sizeof(u_short);

	eor = cp + dlen;

/*
 * Decide whether or not to print this resource record.
 */
	doprint = want_rr(type, filter);

	if (doprint && exclusive && !indomain(rname, name, TRUE))
		doprint = FALSE;

	if (verbose)
		doprintf("%-20s\t%d\t%s\t%s",
			rname, ttl, pr_class(class), pr_type(type));
	else if (ttlprint)
		doprintf("%-20s\t%d\t%s",
			rname, ttl, pr_type(type));
	else
		doprintf("%-20s\t%s",
			rname, pr_type(type));

/*
 * Update resource record statistics for zone listing.
 */
	if (type >= T_FIRST && type <= T_LAST)
		stats[type]++;

/*
 * Save the domain name of an SOA or NS or A record for zone listing.
 */
	if (type == T_A)
		adrname = strcpy(adrnamebuf, rname);
	else if (type == T_NS)
		subname = strcpy(subnamebuf, rname);
	else if (type == T_SOA)
		soaname = strcpy(soanamebuf, rname);

/*
 * Print type specific data, if appropriate.
 */
	switch (type)
	{
	    case T_A:
		switch (class)
		{
		    case C_IN:
			if (dlen == 4)
			{
				bcopy((char *)cp, (char *)&inaddr, sizeof(inaddr));
				doprintf("\t%s", inet_ntoa(inaddr));
				cp += dlen;
			}
			else if (dlen == 7)
			{
				bcopy((char *)cp, (char *)&inaddr, sizeof(inaddr));
				doprintf("\t%s", inet_ntoa(inaddr));
				doprintf(", protocol = %d", cp[4]);
				doprintf(", port = %d", (cp[5] << 8) + cp[6]);
				cp += dlen;
			}
			break;

		    default:
			cp += dlen;
			break;
		}
		break;

	    case T_MX:
		if (check_size(rname, type, cp, msg, eor, sizeof(u_short)) < 0)
			break;
		if (verbose)
			doprintf("\t%ld ", _getshort(cp));
		else
			doprintf("\t");
		cp += sizeof(u_short);

		n = expand(rname, type, cp, msg, eom, dname);
		if (n < 0)
			break;
		doprintf("%s", dname);
		cp += n;
		break;

	    case T_NS:
	    case T_PTR:
	    case T_CNAME:
		n = expand(rname, type, cp, msg, eom, dname);
		if (n < 0)
			break;
		doprintf("\t%s", dname);
		cp += n;
		break;

	    case T_SOA:
		n = expand(rname, type, cp, msg, eom, dname);
		if (n < 0)
			break;
		doprintf("\t%s", dname);
		cp += n;

		n = expand(rname, type, cp, msg, eom, dname);
		if (n < 0)
			break;
		doprintf(" %s", dname);
		cp += n;

		n = 5*sizeof(u_long);
		if (check_size(rname, type, cp, msg, eor, n) < 0)
			break;
		doprintf(" (\n\t\t\t%ld\t;serial (version)", _getlong(cp));
		cp += sizeof(u_long);
		doprintf("\n\t\t\t%ld\t;refresh period", _getlong(cp));
		cp += sizeof(u_long);
		doprintf("\n\t\t\t%ld\t;retry refresh time", _getlong(cp));
		cp += sizeof(u_long);
		doprintf("\n\t\t\t%ld\t;expiration period", _getlong(cp));
		cp += sizeof(u_long);
		doprintf("\n\t\t\t%ld\t;default ttl\n\t\t\t)", _getlong(cp));
		cp += sizeof(u_long);
		break;

	    case T_WKS:
		if (check_size(rname, type, cp, msg, eor, sizeof(u_long)) < 0)
			break;
		bcopy((char *)cp, (char *)&inaddr, sizeof(inaddr));
		doprintf("\t%s", inet_ntoa(inaddr));
		cp += sizeof(u_long);

		if (check_size(rname, type, cp, msg, eor, 1) < 0)
			break;
		n = *cp++;
		protocol = getprotobynumber(n);
		if (protocol)
			doprintf(" %s", protocol->p_name);
		else
			doprintf(" %d", n);

		doprintf(" (");
		n = 0;
		while (cp < eor)
		{
		    register int c;

		    c = *cp++;
		    do
		    {
 			if (c & 0200)
			{
			    if (protocol)
				    service = getservbyport(htons(n), protocol->p_name);
			    else
				    service = NULL;
			    if (service)
				    doprintf(" %s", service->s_name);
			    else
				    doprintf(" %d", n);
			}
 			c <<= 1;
		    } while (++n & 07);
		}
		doprintf(" )");
		break;

	    case T_HINFO:
		if (check_size(rname, type, cp, msg, eor, 1) < 0)
			break;
		n = *cp++;
		if (n > 0)
		{
			doprintf("\t%.*s", n, cp);
			cp += n;
		}

		if (check_size(rname, type, cp, msg, eor, 1) < 0)
			break;
		n = *cp++;
		if (n > 0)
		{
			doprintf("\t%.*s", n, cp);
			cp += n;
		}
		break;

	    case T_MINFO:
		n = expand(rname, type, cp, msg, eom, dname);
		if (n < 0)
			break;
		doprintf("\t%s", dname);
		cp += n;

		n = expand(rname, type, cp, msg, eom, dname);
		if (n < 0)
			break;
		doprintf(" %s", dname);
		cp += n;
		break;

	    case T_MB:
	    case T_MG:
	    case T_MR:
	    case T_MD:
	    case T_MF:
		n = expand(rname, type, cp, msg, eom, dname);
		if (n < 0)
			break;
		doprintf("\t%s", dname);
		cp += n;
		break;

	    case T_TXT:
		while (cp < eor)
		{
			if (check_size(rname, type, cp, msg, eor, 1) < 0)
				break;
			n = *cp++;
			if (n > 0)
			{
				doprintf("\t%.*s", n, cp);
				cp += n;
			}
		}
		break;

#ifdef obsolete
	    case T_TXT:
		if (dlen > 0)
		{
			doprintf("\t%.*s", dlen, cp);
			cp += dlen;
		}
		break;
#endif obsolete

	    case T_UINFO:
		if (dlen > 0)
		{
			doprintf("\t%.*s", dlen, cp);
			cp += dlen;
		}
		break;

	    case T_UID:
	    case T_GID:
		if (dlen == 4)
		{
			doprintf("\t%ld", _getlong(cp));
			cp += dlen;
		}
		break;

	    case T_UNSPEC:
	    case T_NULL:
		cp += dlen;
		break;

	    default:
		doprintf("\t???");
		cp += dlen;
		break;
	}

	doprintf("\n");

/*
 * Save the CNAME alias for cname chain tracing.
 * Save the MR or MG alias for MB chain tracing.
 */
	if ((type == T_CNAME) && n > 0 && cp == eor)
		cname = strcpy(cnamebuf, dname);
	else if ((type == T_MR || type == T_MG) && n > 0 && cp == eor)
		mname = strcpy(mnamebuf, dname);

/*
 * Check if we have reached the exact end of this record.
 */
	if (cp != eor)
	{
		pr_error("size error in %s record for %s, dlen = %d, off by = %d",
			pr_type(type), rname, dlen, (cp - eor));

		/* we believe value of dlen; should perhaps return(NULL) */
		cp = eor;
	}

	return(cp);
}


/*
** SKIP_QR -- Skip the query record in the nameserver answer buffer
** ----------------------------------------------------------------
**
**	Returns:
**		Pointer to position in answer buffer after current record.
**		NULL if there was a format error in the current record.
*/

u_char *
skip_qr(name, cp, msg, eom)
char *name;				/* full name we are querying about */
u_char *cp, *msg, *eom;			/* current positions in answer buf */
{
	char rname[MAXDNAME+1];		/* record name in LHS */
	int type, class;		/* fixed values in query record */
	int n;

	n = expand(name, T_NONE, cp, msg, eom, rname);
	if (n < 0)
		return(NULL);
	cp += n;

	n = 2*sizeof(u_short);
	if (check_size(rname, T_NONE, cp, msg, eom, n) < 0)
		return(NULL);

	type = _getshort(cp);
	cp += sizeof(u_short);

	class = _getshort(cp);
	cp += sizeof(u_short);

#ifdef lint
	if (verbose)
		printf("%-20s\t%s\t%s\n",
			rname, pr_class(class), pr_type(type));
#endif lint

	return(cp);
}


/*
** PR_TYPE -- Return name of resource record type
** ----------------------------------------------
**
**	Returns:
**		Pointer to name of resource record type.
*/

char *
pr_type(type)
int type;				/* resource record type */
{
	static char nbuf[20];

	switch (type)
	{
	    case T_A:      return("A");		/* internet address */
	    case T_NS:     return("NS");	/* authoritative server */
	    case T_MD:     return("MD");	/* mail destination */
	    case T_MF:     return("MF");	/* mail forwarder */
	    case T_CNAME:  return("CNAME");	/* canonical name */
	    case T_SOA:    return("SOA");	/* start of auth zone */
	    case T_MB:     return("MB");	/* mailbox domain name */
	    case T_MG:     return("MG");	/* mail group member */
	    case T_MR:     return("MR");	/* mail rename name */
	    case T_NULL:   return("NULL");	/* null resource record */
	    case T_WKS:    return("WKS");	/* well known service */
	    case T_PTR:    return("PTR");	/* domain name pointer */
	    case T_HINFO:  return("HINFO");	/* host information */
	    case T_MINFO:  return("MINFO");	/* mailbox information */
	    case T_MX:     return("MX");	/* mail routing info */
	    case T_TXT:    return("TXT");	/* descriptive text */

	    case T_UINFO:  return("UINFO");	/* user information */
	    case T_UID:    return("UID");	/* user ident */
	    case T_GID:    return("GID");	/* group ident */
	    case T_UNSPEC: return("UNSPEC");	/* unspecified binary data */

	    case T_AXFR:   return("AXFR");	/* zone transfer */
	    case T_MAILB:  return("MAILB");	/* matches MB/MR/MG/MINFO */
	    case T_MAILA:  return("MAILA");	/* matches MD/MF */
	    case T_ANY:    return("ANY");	/* matches any type */

	    case T_NONE:   return("resource");	/* not yet determined */
	}

	(void) sprintf(nbuf, "%d", type);
	return(nbuf);
}


/*
** PR_CLASS -- Return name of resource record class
** ------------------------------------------------
**
**	Returns:
**		Pointer to name of resource record class.
*/

char *
pr_class(class)
int class;				/* resource record class */
{
	static char nbuf[20];

	switch (class)
	{
	    case C_IN:	   return("IN");	/* internet */
	    case C_CHAOS:  return("CHAOS");	/* chaosnet */
	    case C_HS:	   return("HS");	/* hesiod */
	    case C_ANY:	   return("ANY");	/* any class */
	}

	(void) sprintf(nbuf, "%d", class);
	return(nbuf);
}


/*
** EXPAND -- Expand compressed domain name in a recource record
** ------------------------------------------------------------
**
**	Returns:
**		Number of bytes advanced in answer buffer.
**		-1 if there was a format error.
*/

expand(name, type, cp, msg, eom, namebuf)
char *name;				/* name of resource record */
int type;				/* type of resource record */
u_char *cp, *msg, *eom;			/* current positions in answer buf */
char *namebuf;				/* address of buf to expand name in */
{
	int n;

	n = dn_expand(msg, eom, cp, namebuf, MAXDNAME);
	if (n < 0)
	{
		pr_error("expand error in %s record for %s, offset = %d",
			pr_type(type), name, (cp - msg));
		h_errno = NO_RECOVERY;
		return(-1);
	}

	if (namebuf[0] == '\0')
	{
		namebuf[0] = '.';
		namebuf[1] = '\0';
	}

	return(n);
}


/*
** CHECK_SIZE -- Check whether resource record is of sufficient length
** -------------------------------------------------------------------
**
**	Returns:
**		Requested size if current record is long enough.
**		-1 if current record does not have this many bytes.
*/

check_size(name, type, cp, msg, eor, size)
char *name;				/* name of resource record */
int type;				/* type of resource record */
u_char *cp, *msg, *eor;			/* current positions in answer buf */
int size;				/* required record size remaining */
{
	if (cp + size > eor)
	{
		if (!quiet || type != T_HINFO)
		    pr_error("incomplete %s record for %s, offset = %d",
			pr_type(type), name, (cp - msg));
		h_errno = NO_RECOVERY;
		return(-1);
	}

	return(size);
}


/*
** XALLOC -- Allocate additional memory
** ------------------------------------
**
**	Returns:
**		Pointer to allocated buffer space.
**		Aborts if the requested memory could not be obtained.
*/

char *
xalloc(size)
int size;				/* number of bytes to allocate */
{
	extern char *malloc();
	char *string;

	string = malloc((unsigned)size);
	if (string == NULL)
	{
		fprintf(stderr, "Out of memory\n");
		exit(EX_OSERR);
	}

	return(string);
}


#define MAXNSNAME 10		/* maximum count of nameservers per domain */
#define MAXIPADDR 10		/* maximum count of addresses per nameserver */

char		nsname[MAXNSNAME][MAXDNAME+1];	/* nameserver names */
struct in_addr	ipaddr[MAXNSNAME][MAXIPADDR];	/* nameserver addresses */
int		naddrs[MAXNSNAME];		/* count of addresses */

#define MAXHOSTS 3000		/* maximum count of hostnames per zone */

char		*hostname[MAXHOSTS+1];		/* names of hosts in domain */

#define MAXZONES 2000		/* maximum count of subdomains per zone */


/*
** LIST_DOMAIN -- Basic routine to do complete zone listing and checking
** ---------------------------------------------------------------------
**
**	Returns:
**		TRUE if the requested info was processed successfully.
**		FALSE otherwise.
*/

int total_calls = 0;		/* number of calls for possible zones */
int total_zones = 0;		/* number of zones successfully read */
int total_hosts = 0;		/* number of hosts in all subdomains */

int recursion_level = 0;

list_domain(name)
char *name;				/* name of domain to list */
{
	register int n;
	register int i;
	int nservers;			/* count of nameservers */
	int subzones;			/* count of subdomains */
	char *zonename[MAXZONES+1];	/* names of subdomains */

	total_calls += 1;		/* new attempt */

/*
 * Normalize to not have trailing dot, unless it is the root domain.
 */
	n = strlen(name);
	if (n > 1 && name[n-1] == '.')
		name[n-1] = '\0';

/*
 * Find the nameservers for the given domain.
 */
	nservers = find_servers(name);
	if (nservers < 1)
	{
		fprintf(stderr, "No nameservers for %s found\n", name);
		return(FALSE);
	}

/*
 * Make sure we have an address for at least one nameserver.
 */
	for (n = 0; n < nservers; n++)
		if (naddrs[n] > 0)
			break;
	if (n >= nservers)
	{
		fprintf(stderr, "No addresses found for nameservers\n");
		return(FALSE);
	}

/*
 * Check SOA records at each of the nameservers.
 * Temporarily save our current server info from the resolver data.
 */
	if (checkmode)
	{
		struct state save_res;
		char *save_server;

		save_res = _res;
		save_server = server;

		for (n = 0; n < nservers; n++)
		{
			if (naddrs[n] < 1)
				continue; /* shortcut */

			server = nsname[n];
			for (i = 0; i < MAXNS && i < naddrs[n]; i++)
			{
				_res.nsaddr_list[i].sin_family = AF_INET;
				_res.nsaddr_list[i].sin_port = htons(NAMESERVER_PORT);
				_res.nsaddr_list[i].sin_addr = ipaddr[n][i];
			}
			_res.nscount = i;

			if (check_domain(name) == FALSE)
				hp_error(h_errno, name, T_SOA);
		}

		_res = save_res;
		server = save_server;

		if (!recursive)
			return(TRUE);
	}

/*
 * Ask zone transfer to the nameservers, until one responds.
 */
	for (n = 0; n < nservers; n++)
	{
	    for (i = 0; i < naddrs[n]; i++)
	    {
		if (verbose)
			printf("Trying server %s (%s) ...\n",
				inet_ntoa(ipaddr[n][i]), nsname[n]);

		clear_statistics(); /* reset at each retry */

		subzones = get_zone(name, queryclass, ipaddr[n][i], zonename);
		if (subzones >= 0)
			goto done; /* double break */

		if (h_errno != TRY_AGAIN)
		{
			hp_error(h_errno, name, T_AXFR);
			return(FALSE);
		}
#ifdef notyet
		/* in case nameserver not present */
		if (errno == ECONNREFUSED)
			break;
#endif notyet
	    }
	}
done:
	if (n >= nservers)
	{
		hp_error(h_errno, name, T_AXFR);
		fprintf(stderr, "No nameservers for %s responded\n", name);
		return(FALSE);
	}

/*
 * Print resource record statistics if so requested.
 */
	if (statistics)
		print_statistics(name, querytype);

/*
 * The names of the hosts were allocated dynamically.
 */
	for (n = 0; hostname[n]; n++)
		(void) free(hostname[n]);

	if (verbose || statistics || hostmode)
		printf("Found %d host%s within %s\n",
			n, n == 1 ? "" : "s", name);

	total_hosts += n;		/* update total number of hosts */
	total_zones += 1;		/* update total zones processed */

/*
 * Do recursion on subdomains if requested and any were found.
 */
	if (recursive && (recursion_level < recursive))
	{
		if (verbose || statistics)
			printf("Found %d subdomain%s within %s\n",
				subzones, subzones == 1 ? "" : "s", name);

		for (n = 0; n < subzones; n++)
		{
			if (verbose || statistics || checkmode)
				printf("\n");

			if (verbose)
				printf("Entering subdomain %s\n", zonename[n]);

			recursion_level++;
			(void) list_domain(zonename[n]);
			recursion_level--;
		}
	}

/*
 * The names of the subdomains were allocated dynamically.
 */
	for (n = 0; zonename[n]; n++)
		(void) free(zonename[n]);

	if (recursive && (recursion_level == 0))
	{
		if (verbose || statistics || checkmode)
			printf("\n");

		if (verbose || statistics || hostmode)
			printf("Encountered %d host%s in %d domain%s within %s\n",
				total_hosts, total_hosts == 1 ? "" : "s",
				total_zones, total_zones == 1 ? "" : "s",
				name);

		if (verbose || statistics || checkmode)
			printf("Processed %d domain%s out of %d attempt%s\n",
				total_zones, total_zones == 1 ? "" : "s",
				total_calls, total_calls == 1 ? "" : "s");
	}

	return(TRUE);
}


/*
** FIND_SERVERS -- Fetch names and addresses of authoritative servers
** ------------------------------------------------------------------
**
**	Returns:
**		The number of servers found.
**		0 if no servers could be determined successfully.
**
**	Inputs:
**		The global variable server, if set, contains the name
**		of the explicit server to be contacted.
**		The global variable primary, if set, indicates that
**		we must use the primary nameserver for the domain.
**
**	Outputs:
**		Names are stored in the nsname[] database.
**		Addresses are stored in the ipaddr[] database.
**		Address counts are stored in the naddrs[] database.
*/

find_servers(name)
char *name;				/* name of domain to find servers for */
{
	struct hostent *hp;
	register int n;
	register int i;
	int nservers;			/* count of nameservers */

/*
 * Use the explicit server if given on the command line.
 * Its addresses are stored in the resolver state struct.
 * This server may not be authoritative for the given domain.
 */
	if (server)
	{
		(void) strcpy(nsname[0], server);
		for (i = 0; i < MAXIPADDR && i < _res.nscount; i++)
			ipaddr[0][i] = _res.nsaddr_list[i].sin_addr;
		naddrs[0] = i;

		nservers = 1;
		return(nservers);
	}

/*
 * Fetch primary nameserver info if so requested.
 * Get its name from the SOA record for the domain,
 * and do a regular host lookup to fetch its addresses.
 */
	if (primary)
	{
		char *primaryname;

		primaryname = get_primary(name);
		if (primaryname == NULL)
		{
			hp_error(h_errno, name, T_SOA);
			return(0);
		}

		hp = gethostbyname(primaryname);
		if (hp == NULL)
		{
			hp_error(h_errno, primaryname, T_A);
			return(0);
		}

		(void) strcpy(nsname[0], hp->h_name);
		for (i = 0; i < MAXIPADDR && hp->h_addr_list[i]; i++)
			ipaddr[0][i] = *(struct in_addr *)hp->h_addr_list[i];
		naddrs[0] = i;

		if (verbose)
			printf("Found %d address%s for %s\n",
				naddrs[0], naddrs[0] == 1 ? "  " : "es",
				nsname[0]);

		nservers = 1;
		return(nservers);
	}

/*
 * Otherwise we have to find the nameservers for the domain.
 */
	nservers = get_servers(name);
	if (nservers < 1)
	{
		hp_error(h_errno, name, T_NS);
		return(0);
	}

/*
 * Usually we'll get addresses for all the servers in the additional
 * info section.  But in case we don't, look up their addresses.
 */
	for (n = 0; n < nservers; n++)
	{
	    if (naddrs[n] == 0)
	    {
		hp = gethostbyname(nsname[n]);
		if (hp != NULL)
		{
			for (i = 0; i < MAXIPADDR && hp->h_addr_list[i]; i++)
				ipaddr[n][i] = *(struct in_addr *)hp->h_addr_list[i];
			naddrs[n] = i;
		}

		if (verbose)
			printf("Found %d address%s for %s by extra query\n",
				naddrs[n], naddrs[n] == 1 ? "  " : "es",
				nsname[n]);
	    }
	    else
	    {
		if (verbose)
			printf("Found %d address%s for %s\n",
				naddrs[n], naddrs[n] == 1 ? "  " : "es",
				nsname[n]);
	    }
	}

	return(nservers);
}


/*
** GET_SERVERS -- Fetch names and addresses of authoritative servers
** -----------------------------------------------------------------
**
**	Returns:
**		The number of servers found.
**		0 if no servers could be determined successfully.
**
**	Side effects:
**		Names are stored in the nsname[] database.
**		Addresses are stored in the ipaddr[] database.
**		Address counts are stored in the naddrs[] database.
*/

get_servers(name)
char *name;				/* name of domain to find servers for */
{
	querybuf answer;
	int anslen;
	int nservers;			/* count of nameservers */

	if (verbose)
		printf("Finding nameservers for %s ...\n", name);

	anslen = get_info(&answer, name, T_NS, queryclass);
	if (anslen < 0)
		return(0);

	if (verbose > 1)
		(void) print_info(&answer, anslen, name, T_ANY);

	nservers = get_nsinfo(&answer, anslen, name);
	return(nservers);
}


/*
** GET_NSINFO -- Extract nameserver data from nameserver answer buffer
** -------------------------------------------------------------------
**
**	Returns:
**		The number of servers found.
**		0 if no servers could be determined successfully.
**
**	Outputs:
**		Names are stored in the nsname[] database.
**		Addresses are stored in the ipaddr[] database.
**		Address counts are stored in the naddrs[] database.
*/

get_nsinfo(answerbuf, answerlen, name)
querybuf *answerbuf;			/* address of answer buffer */
int answerlen;				/* length of answer buffer */
char *name;				/* name of domain to find servers for */
{
	HEADER *bp;
	int qdcount, ancount, nscount, arcount;
	int rrcount;
	u_char *msg, *eom, *cp;
	register int i;
	int nservers = 0;		/* count of nameservers */

	bp = (HEADER *)answerbuf;
	qdcount = ntohs(bp->qdcount);
	ancount = ntohs(bp->ancount);
	nscount = ntohs(bp->nscount);
	arcount = ntohs(bp->arcount);

	msg = (u_char *)answerbuf;
	eom = (u_char *)answerbuf + answerlen;
	cp  = (u_char *)answerbuf + sizeof(HEADER);

	while (qdcount > 0 && cp < eom)
	{
		cp = skip_qr(name, cp, msg, eom);
		if (cp == NULL)
			return(0);
		qdcount--;
	}

/*
 * If the answer is authoritative, the names are found in the
 * answer section, and the nameserver section is empty.
 * If not, there may be duplicate names in both sections.
 * Addresses are found in the additional info section both cases.
 */
	rrcount = ancount + nscount + arcount;
	while (rrcount > 0 && cp < eom)
	{
		char rname[MAXDNAME+1];
		char dname[MAXDNAME+1];
		int type, class, ttl, dlen;
		int n;
		struct in_addr inaddr;

		n = expand(name, T_NONE, cp, msg, eom, rname);
		if (n < 0)
			break;
		cp += n;

		n = 3*sizeof(u_short) + sizeof(u_long);
		if (check_size(rname, T_NONE, cp, msg, eom, n) < 0)
			break;

		type = _getshort(cp);
		cp += sizeof(u_short);

		class = _getshort(cp);
		cp += sizeof(u_short);

		ttl = _getlong(cp);
		cp += sizeof(u_long);

		dlen = _getshort(cp);
		cp += sizeof(u_short);

#ifdef lint
		if (verbose)
			printf("%-20s\t%d\t%s\t%s\n",
				rname, ttl, pr_class(class), pr_type(type));
#endif lint

		if ((type == T_NS) && sameword(rname, name))
		{
			n = expand(rname, type, cp, msg, eom, dname);
			if (n < 0)
				break;

			for (i = 0; i < nservers; i++)
				if (sameword(nsname[i], dname))
					break;  /* duplicate */
			if (i >= nservers && nservers < MAXNSNAME)
			{
				(void) strcpy(nsname[nservers], dname);
				naddrs[nservers] = 0;
				nservers++;
			}
		}
		else if ((type == T_A) && dlen == 4)
		{
			for (i = 0; i < nservers; i++)
				if (sameword(nsname[i], rname))
					break;	/* found */
			if (i < nservers && naddrs[i] < MAXIPADDR)
			{
				bcopy((char *)cp, (char *)&inaddr, sizeof(inaddr));
				ipaddr[i][naddrs[i]] = inaddr;
				naddrs[i]++;
			}
		}

		cp += dlen;
		rrcount--;
	}

	return(nservers);
}


/*
** GET_ZONE -- Perform a zone transfer from server at specific address
** -------------------------------------------------------------------
**
**	Returns:
**		The number of subdomains found in the given domain.
**		-1 if an error occurred (h_errno is set appropriately).
**
**	Side effects:
**		Stores list of subdomains found in zonename[].
**		Stores list of hostnames  found in hostname[].
**		Updates resource record statistics in stats[].
**		This array must have been cleared before.
*/

get_zone(name, class, inaddr, zonename)
char *name;				/* name of domain to do zone xfer for */
int class;				/* specific resource record class */
struct in_addr inaddr;			/* address of server to be queried */
char *zonename[MAXZONES+1];		/* names of subdomains found in zone */
{
	querybuf query;
	querybuf answer;
	HEADER *bp;
	int ancount;
	int n;
	int sock;
	struct sockaddr_in sin;
	register int i;
	int nrecords = 0;		/* number of records processed */
	int soacount = 0;		/* count of SOA records */
	int nzones = 0;			/* count of subdomains */
	int nhosts = 0;			/* count of hostnames */

/*
 * Construct query, and connect to the given server.
 */
	errno = 0;

	n = res_mkquery(QUERY, name, class, T_AXFR,
			(char *)NULL, 0, (char *)NULL,
			(char *)&query, sizeof(querybuf));
	if (n < 0)
	{
		if (debug)
			fprintf(stderr, "res_mkquery failed\n");
		h_errno = NO_RECOVERY;
		return(-1);
	}

	if (debug)
	{
		printf("get_zone()\n");
		p_query((char *)&query);
	}

	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock < 0)
	{
		perror("socket");
		h_errno = NO_RECOVERY;
		return(-1);
	}

	sin.sin_family = AF_INET;
	sin.sin_port = htons(NAMESERVER_PORT);
	sin.sin_addr = inaddr;

	if (connsock(sock, &sin, sizeof(sin)) < 0)
	{
		if (verbose)
			perror("connect");
		(void) close(sock);
		h_errno = TRY_AGAIN;
		return(-1);
	}

	if (verbose)
		printf("Asking zone transfer for %s ...\n", name);

/*
 * Send the query buffer.
 */
	if (writemsg(sock, (char *)&query, n) < 0)
	{
		(void) close(sock);
		h_errno = TRY_AGAIN;
		return(-1);
	}

/*
 * Process all incoming records, each record in a separate packet.
 */
	while ((n = readmsg(sock, (char *)&answer, sizeof(querybuf))) != 0)
	{
		if (n < 0)
		{
			(void) close(sock);
			h_errno = TRY_AGAIN;
			return(-1);
		}

		if (n < sizeof(HEADER))
		{
			pr_error("answer length %d too short", n);
			(void) close(sock);
			h_errno = TRY_AGAIN;
			return(-1);
		}

		if (debug > 1)
		{
			printf("got answer:\n");
			p_query((char *)&answer);
		}

	/*
	 * Analyze the contents of the answer and check for errors.
	 */
		bp = (HEADER *)&answer;
		ancount = ntohs(bp->ancount);

		if (bp->rcode != NOERROR || ancount == 0)
		{
			if (verbose)
				print_status(&answer);

			switch (bp->rcode)
			{
			    case NXDOMAIN:
				/* check if it's an authoritative answer */
				if (bp->aa)
					h_errno = HOST_NOT_FOUND;
				else
					h_errno = TRY_AGAIN;
				break;

			    case NOERROR:
				h_errno = NO_DATA;
				break;

			    default:
				h_errno = TRY_AGAIN;
				break;
			}

			(void) close(sock);
			return(-1);
		}

		h_errno = 0;

	/*
	 * Valid packet received. Print contents if appropriate.
	 */
		nrecords++;
		soaname = NULL;
		subname = NULL;
		adrname = NULL;

		(void) print_info(&answer, n, name, querytype);

	/*
	 * Terminate upon the second SOA record for this domain.
	 */
		if (soaname && sameword(soaname, name))
			if (soacount++)
				break;

		/* the nameserver balks on this one */
		if (soaname && !sameword(soaname, name))
		    if (!quiet)
			pr_error("extraneous SOA record for %s within %s", soaname, name);

	/*
	 * Save encountered subdomain name for recursive listing.
	 */
		if (subname && samedomain(subname, name, FALSE))
		{
			for (i = 0; i < nzones; i++)
				if (sameword(zonename[i], subname))
					break;	/* duplicate */
			if (i >= nzones && nzones < MAXZONES)
			{
				zonename[nzones] = newstr(subname);
				nzones++;

				if (nzones == MAXZONES)
					pr_error("maximum number of %d subdomains reached", nzones);
			}
		}

#ifdef notyet
		/* warn about strange subdomains */
		if (subname && !samedomain(subname, name, TRUE))
		    if (!quiet)
			pr_error("extraneous NS record for %s within %s", subname, name);
#endif notyet

		/* not sure whether this is illegal or not */
		if (subname && !indomain(subname, name, TRUE))
		    if (!quiet)
			pr_error("extraneous NS record for %s within %s", subname, name);

	/*
	 * Save encountered name of A record for hostname count.
	 */
		if (adrname && indomain(adrname, name, FALSE))
		{
			for (i = 0; i < nhosts; i++)
				if (sameword(hostname[i], adrname))
					break;	/* duplicate */
			if (i >= nhosts && nhosts < MAXHOSTS)
			{
				hostname[nhosts] = newstr(adrname);
				nhosts++;

				if (nhosts == MAXHOSTS)
					pr_error("maximum number of %d hostnames reached", nhosts);
			}
		}

		/* check for unauthoritative glue records */
		if (adrname && !samedomain(adrname, name, TRUE))
		    if (!quiet)
			pr_error("extraneous glue record for %s within %s", adrname, name);
	}

	(void) close(sock);

/*
 * Do extra check for hostnames also defined as subdomains.
 * They may have been defined in the child domain, and crept in
 * the parent domain, or may have been defined as glue records.
 * This is not necessarily an error, but the hostname count may
 * be actually wrong. Leave it in for the time being.
 */
	for (n = 0; n < nhosts; n++)
	{
	    for (i = 0; i < nzones; i++)
	    {
		if (sameword(hostname[n], zonename[i]))
		    if (!quiet)
			pr_error("extraneous A record for %s within %s", hostname[n], name);
	    }
	}

/*
 * End of zone transfer at second SOA record or zero length read.
 */
	if (verbose)
		printf("Transfer complete, %d records received for %s\n", nrecords, name);

	hostname[nhosts] = NULL;	/* terminate list of hostnames */
	zonename[nzones] = NULL;	/* terminate list of subdomains */

	return(nzones);
}


/*
** SOCKETIO -- Miscellaneous socket I/O routines
** ---------------------------------------------
*/

writemsg(sock, buf, bufsize)
int sock;
char *buf;
int bufsize;
{
	u_short len;

	len = htons(bufsize);

	if (write(sock, (char *)&len, sizeof(len)) != sizeof(len) ||
	    write(sock, buf, bufsize) != bufsize)
	{
		perror("write");
		return(-1);
	}

	return(bufsize);
}


readmsg(sock, buf, bufsize)
int sock;
char *buf;
int bufsize;
{
	u_short len;
	char *buffer;
	int buflen;
	int n;

/*
 * Read the length of answer buffer.
 */
	buffer = (char *)&len;
	buflen = sizeof(len);

	while (buflen > 0 && (n = readsock(sock, buffer, buflen)) > 0)
	{
		buffer += n;
		buflen -= n;
	}

	if (buflen != 0)
	{
		perror("read answer length");
		return(-1);
	}

/*
 * Terminate if length is zero. Check for buffer overflow.
 */
	len = ntohs(len);
	if (len == 0)
		return(0);

	if (len > bufsize)
	{
		pr_error("answer length %d too long", len);
		return(-1);
	}

/*
 * Read the answer buffer itself.
 */
	buffer = buf;
	buflen = len;

	while (buflen > 0 && (n = readsock(sock, buffer, buflen)) > 0)
	{
		buffer += n;
		buflen -= n;
	}

	if (buflen != 0)
	{
		perror("read answer");
		return(-1);
	}

	return(len);
}


readsock(sock, buffer, buflen)
int sock;
char *buffer;
int buflen;
{
	int n;
	fd_set fds;
	struct timeval wait;

	wait.tv_sec = 60;
	wait.tv_usec = 0;

	/* FD_ZERO(&fds); */
	bzero((char *)&fds, sizeof(fds));
	FD_SET(sock, &fds);

/*
 * Wait for the arrival of data, or timeout.
 */
	n = select(FD_SETSIZE, &fds, (fd_set *)0, (fd_set *)0, &wait);
	if (n <= 0)
	{
		if (n == 0)
			errno = ETIMEDOUT;
		return(-1);
	}

/*
 * Fake an error if nothing was actually read.
 */
	n = recv(sock, buffer, buflen, 0);
	if (n == 0)
		errno = ECONNRESET;
	return(n);
}


jmp_buf timeout;

timer()
{
	longjmp(timeout, 1);
}


connsock(sock, addr, addrlen)
int sock;
struct sockaddr_in *addr;
int addrlen;
{
	if (setjmp(timeout) != 0)
	{
		errno = ETIMEDOUT;
		(void) alarm((unsigned)0);
		return(-1);
	}

	(void) signal(SIGALRM, timer);
	(void) alarm((unsigned)_res.retrans);

	if (connect(sock, addr, addrlen) < 0)
	{
		if (errno == EINTR)
			errno = ETIMEDOUT;
		(void) alarm((unsigned)0);
		return(-1);
	}

	(void) alarm((unsigned)0);
	return(0);
}


struct soa_data
{
	char	sname[MAXDNAME+1];	/* name of primary server */
	char	mname[MAXDNAME+1];	/* name of hostmaster */
	int	serial;			/* serial (version) number */
	int	refresh;		/* refresh time in seconds */
	int	retry;			/* refresh retry time in seconds */
	int	expire;			/* expiration time in seconds */
	int	defttl;			/* default time_to_live */
};

struct soa_data soa;			/* buffer to store soa data */


/*
** GET_PRIMARY -- Fetch name of primary nameserver for a domain
** ------------------------------------------------------------
**
**	Returns:
**		Pointer to the name of the primary server, if found.
**		NULL if the server could not be determined.
*/

char *
get_primary(name)
char *name;				/* name of domain to get soa for */
{
	querybuf answer;
	int anslen;

	if (verbose)
		printf("Finding primary nameserver for %s ...\n", name);

	anslen = get_info(&answer, name, T_SOA, queryclass);
	if (anslen < 0)
		return(NULL);

	if (verbose > 1)
		(void) print_info(&answer, anslen, name, T_ANY);

	soaname = NULL;
	(void) get_soainfo(&answer, anslen, name);
	if (soaname == NULL)
		return(NULL);

	return(soa.sname);
}


/*
** CHECK_DOMAIN -- Fetch and analyse SOA record of a domain
** --------------------------------------------------------
**
**	Returns:
**		TRUE if the SOA record was found at the given server.
**		FALSE otherwise.
**
**	Inputs:
**		The global variable server must contain the name
**		of the server that was queried.
*/

check_domain(name)
char *name;				/* name of domain to get soa for */
{
	querybuf answer;
	int anslen;

	if (verbose)
		printf("Checking SOA for %s at server %s\n", name, server);
	else
		printf("%s (%s)\n", name, server);

	anslen = get_info(&answer, name, T_SOA, queryclass);
	if (anslen < 0)
		return(FALSE);

	if (verbose > 1)
		(void) print_info(&answer, anslen, name, T_ANY);

	soaname = NULL;
	(void) get_soainfo(&answer, anslen, name);
	if (soaname == NULL)
		return(FALSE);

	printf("%s\t%s\t(%d %d %d %d %d)\n", soa.sname, soa.mname,
		soa.serial, soa.refresh, soa.retry, soa.expire, soa.defttl);

	(void) check_soa(name);
	return(TRUE);
}


/*
** GET_SOAINFO -- Extract SOA data from nameserver answer buffer
** -------------------------------------------------------------
**
**	Returns:
**		NOERROR if the SOA record was found successfully.
**		FORMERR otherwise.
**
**	Outputs:
**		The global struct soa is filled with the soa data.
**
**	Side effects:
**		Sets soaname if this is a valid SOA record.
**		This variable must have been cleared before calling
**		get_soainfo() and may be checked afterwards.
*/

get_soainfo(answerbuf, answerlen, name)
querybuf *answerbuf;			/* address of answer buffer */
int answerlen;				/* length of answer buffer */
char *name;				/* name of domain to get soa for */
{
	HEADER *bp;
	int qdcount, ancount;
	u_char *msg, *eom, *cp;

	bp = (HEADER *)answerbuf;
	qdcount = ntohs(bp->qdcount);
	ancount = ntohs(bp->ancount);

	msg = (u_char *)answerbuf;
	eom = (u_char *)answerbuf + answerlen;
	cp  = (u_char *)answerbuf + sizeof(HEADER);

	while (qdcount > 0 && cp < eom)
	{
		cp = skip_qr(name, cp, msg, eom);
		if (cp == NULL)
			return(FORMERR);
		qdcount--;
	}

	if (qdcount)
	{
		pr_error("invalid qdcount in response");
		return(FORMERR);
	}

/*
 * Check answer section only.
 * The nameserver section may contain the nameservers for the domain,
 * and the additional section their addresses, but not guaranteed.
 */
	while (ancount > 0 && cp < eom)
	{
		char rname[MAXDNAME+1];
		int type, class, ttl, dlen;
		int n;
		u_char *eor;

		n = expand(name, T_NONE, cp, msg, eom, rname);
		if (n < 0)
			return(FORMERR);
		cp += n;

		n = 3*sizeof(u_short) + sizeof(u_long);
		if (check_size(rname, T_NONE, cp, msg, eom, n) < 0)
			return(FORMERR);

		type = _getshort(cp);
		cp += sizeof(u_short);

		class = _getshort(cp);
		cp += sizeof(u_short);

		ttl = _getlong(cp);
		cp += sizeof(u_long);

		dlen = _getshort(cp);
		cp += sizeof(u_short);

		eor = cp + dlen;

#ifdef lint
		if (verbose)
			printf("%-20s\t%d\t%s\t%s\n",
				rname, ttl, pr_class(class), pr_type(type));
#endif lint

		switch (type)
		{
		    case T_SOA:
			n = expand(rname, type, cp, msg, eom, soa.sname);
			if (n < 0)
				break;
			cp += n;

			n = expand(rname, type, cp, msg, eom, soa.mname);
			if (n < 0)
				break;
			cp += n;

			n = 5*sizeof(u_long);
			if (check_size(rname, type, cp, msg, eor, n) < 0)
				break;
			soa.serial = _getlong(cp);
			cp += sizeof(u_long);
			soa.refresh = _getlong(cp);
			cp += sizeof(u_long);
			soa.retry = _getlong(cp);
			cp += sizeof(u_long);
			soa.expire = _getlong(cp);
			cp += sizeof(u_long);
			soa.defttl = _getlong(cp);
			cp += sizeof(u_long);

			/* valid complete soa record found */
			soaname = strcpy(soanamebuf, rname);
			break;

		    default:
			cp += dlen;
			break;
		}

		if (cp != eor)
		{
			pr_error("size error in %s record for %s, dlen = %d, off by = %d",
				pr_type(type), rname, dlen, (cp - eor));
			return(FORMERR);
		}

		ancount--;
	}

	if (ancount)
	{
		pr_error("invalid ancount in response");
		return(FORMERR);
	}

	return(NOERROR);
}


/*
** CHECK_SOA -- Compare new SOA record with previous SOA record
** ------------------------------------------------------------
**
**	Returns:
**		None.
**
**	Inputs:
**		The global variable server must contain the name
**		of the server that was queried.
**		The global struct soa must contain the soa data.
*/

check_soa(name)
char *name;				/* name of domain to check soa for */
{
	static char *oldname = NULL;	/* previous name of domain */
	static char *oldserver = NULL;	/* previous name of server */
	static struct soa_data oldsoa;	/* previous soa data */

	if (oldname && !sameword(name, oldname))
		oldname = NULL;

	if (oldname)
	{
		if (soa.serial != oldsoa.serial)
			pr_error("%s has different serial than %s",
				server, oldserver);
		if (!sameword(soa.sname, oldsoa.sname))
			pr_error("%s has different primary than %s",
				server, oldserver);
		if (!sameword(soa.mname, oldsoa.mname))
			pr_error("%s has different hostmaster than %s",
				server, oldserver);
	}

	oldname = name;
	oldserver = server;
	oldsoa = soa;
}


#define MAXADDRS 35		/* maximum address count from gethostnamadr.c */


/*
** CHECK_ADDR -- Check if reverse address mappings revert to host
** --------------------------------------------------------------
**
**	Returns:
**		TRUE if all addresses of host map back to host.
**		FALSE otherwise.
*/

check_addr(name)
char *name;				/* host name to check addresses for */
{
	struct hostent *hp;
	register int i;
	struct in_addr inaddr[MAXADDRS];
	char hnamebuf[MAXDNAME+1];
	char *hname;
	int naddr;
	int matched;

/*
 * Look up the specified host to fetch its addresses.
 */
	hp = gethostbyname(name);
	if (hp == NULL)
	{
		hp_error(h_errno, name, T_A);
		return(FALSE);
	}

	hname = strcpy(hnamebuf, hp->h_name);
	for (i = 0; i < MAXADDRS && hp->h_addr_list[i]; i++)
		inaddr[i] = *(struct in_addr *)hp->h_addr_list[i];
	naddr = i;

	if (verbose)
		printf("Found %d address%s for %s\n",
			naddr, naddr == 1 ? "" : "es", hname);

/*
 * Map back the addresses found, and check if they revert to host.
 */
	for (matched = 0, i = 0; i < naddr; i++)
	{
		char *iname = inet_ntoa(inaddr[i]);
		u_long addr = inaddr[i].s_addr;

		if (verbose)
			printf("Checking %s address %s\n", hname, iname);

		hp = gethostbyaddr((char *)&addr, sizeof(addr), AF_INET);
		if (hp == NULL)
			hp_error(h_errno, iname, T_PTR);
		else if (!sameword(hp->h_name, hname))
			pr_error("address %s of %s maps to %s", iname, hname, hp->h_name);
		else
			matched++;
	}

	return(matched == naddr ? TRUE : FALSE);
}


/*
** CHECK_NAME -- Check if address belongs to host addresses
** --------------------------------------------------------
**
**	Returns:
**		TRUE if given address was found among host addresses.
**		FALSE otherwise.
*/

check_name(addr)
u_long addr;				/* address of host to check */
{
	struct hostent *hp;
	register int i;
	struct in_addr inaddr;
	char hnamebuf[MAXDNAME+1];
	char *hname;
	char inamebuf[MAXDNAME+1];
	char *iname;
	int matched;

/*
 * Check if the address is registered by fetching its hostname.
 */
	inaddr.s_addr = addr;
	iname = strcpy(inamebuf, inet_ntoa(inaddr));

	hp = gethostbyaddr((char *)&addr, sizeof(addr), AF_INET);
	if (hp == NULL)
	{
		hp_error(h_errno, iname, T_PTR);
		return(FALSE);
	}

	hname = strcpy(hnamebuf, hp->h_name);

	if (verbose)
		printf("Address %s maps to %s\n", iname, hname);

/*
 * Lookup the hostname found to fetch its addresses.
 * Check if the given address is listed among the known addresses.
 */
	hp = gethostbyname(hname);
	if (hp == NULL)
	{
		hp_error(h_errno, hname, T_A);
		return(FALSE);
	}

	for (matched = 0, i = 0; hp->h_addr_list[i]; i++)
	{
		inaddr = *(struct in_addr *)hp->h_addr_list[i];

		if (verbose)
			printf("Checking %s address %s\n", hname, inet_ntoa(inaddr));

		if (inaddr.s_addr == addr)
			matched++;
	}

	if (!matched)
		pr_error("address %s does not belong to %s", iname, hname);

	return(matched ? TRUE : FALSE);
}
